unit UDoodle;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  Windows,
{$ELSE}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Menus;

type

  { TDoodle }

  TDoodle = class(TForm)
    MainMenu1: TMainMenu;
    mFile: TMenuItem;
    mFileBar2: TMenuItem;
    mFileExit: TMenuItem;
    mFileNew: TMenuItem;
    mImage: TMenuItem;
    mImageClear: TMenuItem;
    mFileOpen: TMenuItem;
    mFileSave: TMenuItem;
    mFileSaveAs: TMenuItem;
    mFileClose: TMenuItem;
    mFileCloseAll: TMenuItem;
    mFileBar1: TMenuItem;
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure mImageClearClick(Sender: TObject);
    procedure mFileNewClick(Sender: TObject);
    procedure mFileOpenClick(Sender: TObject);
    procedure mFileExitClick(Sender: TObject);
    procedure mFileSaveClick(Sender: TObject);
    procedure mFileSaveAsClick(Sender: TObject);
    procedure mFileCloseClick(Sender: TObject);
    procedure mFileCloseAllClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    DoodleBmp: TBitmap;
    Drawing: Boolean;
    Modified: Boolean;
  public
    { Public declarations }
  end;

var
  Doodle: TDoodle;
  Counter: Integer;

implementation

uses UMain;

{$IFnDEF FPC}
  {$R *.dfm}
{$ELSE}
  {$R *.lfm}
{$ENDIF}

procedure TDoodle.FormResize(Sender: TObject);
var
  NewBmp: TBitmap;
begin
  if (not Assigned(DoodleBmp)) or
     (DoodleBmp.Width < ClientWidth) or
     (DoodleBmp.Height < ClientHeight) then begin
    NewBmp:=TBitmap.Create;
    {$IFDEF FPC}
    NewBmp.PixelFormat:=pf32Bit;
    {$ENDIF}
    NewBmp.Width:=ClientWidth;
    NewBmp.Height:=ClientHeight;
    NewBmp.Canvas.Rectangle(-10, -10, ClientWidth + 10, ClientHeight + 1);
    if Assigned(DoodleBmp) then begin
      NewBmp.Canvas.Draw(0, 0, DoodleBmp);
      DoodleBmp.Free;
    end;
    DoodleBmp:=NewBmp;
  end;
end;

procedure TDoodle.FormDestroy(Sender: TObject);
begin
  if Assigned(DoodleBmp) then DoodleBmp.Free;
  Main.EnableDoodleControls(False);
end;

procedure TDoodle.FormPaint(Sender: TObject);
begin
  Canvas.Draw(0, 0, DoodleBmp);
end;

procedure TDoodle.FormCreate(Sender: TObject);
begin
  FormResize(Sender);
  Inc(Counter);
  Caption:='Untitled' + IntToStr(Counter);
end;

procedure TDoodle.FormActivate(Sender: TObject);
begin
  Main.UpdateActiveDoodle;
end;

procedure TDoodle.FormDeactivate(Sender: TObject);
begin
  Main.UpdateActiveDoodle;
end;

procedure TDoodle.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button=mbLeft then begin
    Drawing:=True;
    Modified:=True;
    with DoodleBmp.Canvas do begin
      Pen.Color:=Main.plColor.Color;
      Pen.Width:=Main.tbWidth.Position;
      MoveTo(X, Y);
      LineTo(X+1, Y);
      MoveTo(X, Y);
    end;
    Canvas.Draw(0, 0, DoodleBmp);
  end;
end;

procedure TDoodle.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if Drawing then begin
    DoodleBmp.Canvas.LineTo(X, Y);
    Canvas.Draw(0, 0, DoodleBmp);
  end;
end;

procedure TDoodle.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button=mbLeft then Drawing:=False;
end;

procedure TDoodle.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action:=caFree;
end;

procedure TDoodle.mImageClearClick(Sender: TObject);
begin
  DoodleBmp.Canvas.Rectangle(-10, -10, DoodleBmp.Width + 10, DoodleBmp.Height + 10);
  Canvas.Draw(0, 0, DoodleBmp);
end;

procedure TDoodle.mFileNewClick(Sender: TObject);
begin
  Main.mFileNew.Click;
end;

procedure TDoodle.mFileOpenClick(Sender: TObject);
begin
  Main.mFileOpen.Click;
end;

procedure TDoodle.mFileExitClick(Sender: TObject);
begin
  Main.mFileExit.Click;
end;

procedure TDoodle.mFileSaveClick(Sender: TObject);
begin
  ShowMessage('Assume saving file ' + Caption + ' is implemented');
end;

procedure TDoodle.mFileSaveAsClick(Sender: TObject);
begin
  ShowMessage('Assume saving file ' + Caption + ' under a different name is implemented');
end;

procedure TDoodle.mFileCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TDoodle.mFileCloseAllClick(Sender: TObject);
begin
  Main.CloseAllWindows;
end;

procedure TDoodle.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if Modified then begin
     CanClose:=MessageDlg(Caption + ' has been modified. Do you want to close?',
       mtConfirmation, [mbYes, mbNo], 0)=mrYes;
  end else CanClose:=True;
end;

end.
