unit UMain;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  Windows,
{$ELSE}
  LCLIntf, LCLType,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, StdCtrls, Buttons, ExtCtrls;

type

  { TMain }

  TMain = class(TForm)
    MainMenu1: TMainMenu;
    mFile: TMenuItem;
    mFileNew: TMenuItem;
    mFileOpen: TMenuItem;
    mFileBar1: TMenuItem;
    mFileExit: TMenuItem;
    mHelp: TMenuItem;
    mHelpAbout: TMenuItem;
    StatusBar1: TStatusBar;
    plToolBar: TPanel;
    sbNew: TSpeedButton;
    sbOpen: TSpeedButton;
    sbSave: TSpeedButton;
    plPenOptions: TPanel;
    Label1: TLabel;
    plColor: TPanel;
    Label2: TLabel;
    tbWidth: TTrackBar;
    tmrFirstTimeInit: TTimer;
    ColorDialog1: TColorDialog;
    mWindow: TMenuItem;
    mWindowTile: TMenuItem;
    mWindowCascade: TMenuItem;
    mWindowArrangeIcons: TMenuItem;
    mWindowBar1: TMenuItem;
    sbPrevious: TSpeedButton;
    sbNext: TSpeedButton;
    mView: TMenuItem;
    mViewToolbar: TMenuItem;
    mViewStatus: TMenuItem;
    SpeedButton1: TSpeedButton;
    procedure mHelpAboutClick(Sender: TObject);
    procedure sbNewClick(Sender: TObject);
    procedure sbOpenClick(Sender: TObject);
    procedure mFileNewClick(Sender: TObject);
    procedure tmrFirstTimeInitTimer(Sender: TObject);
    procedure plColorClick(Sender: TObject);
    procedure mWindowTileClick(Sender: TObject);
    procedure mWindowCascadeClick(Sender: TObject);
    procedure mWindowArrangeIconsClick(Sender: TObject);
    procedure sbPreviousClick(Sender: TObject);
    procedure sbNextClick(Sender: TObject);
    procedure mFileExitClick(Sender: TObject);
    procedure mFileOpenClick(Sender: TObject);
    procedure sbSaveClick(Sender: TObject);
    procedure mViewToolbarClick(Sender: TObject);
    procedure mViewStatusClick(Sender: TObject);
    procedure plToolBarClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  private
    procedure WindowMenuItemClick(Sender: TObject);
  public
    procedure EnableDoodleControls(Flag: Boolean);
    procedure UpdateActiveDoodle;
    procedure ClearWindowMenu;
    procedure UpdateWindowMenu;
    procedure CloseAllWindows;
  end;

var
  Main: TMain;

implementation

uses UDoodle;

{$IFnDEF FPC}
  {$R *.dfm}
{$ELSE}
  {$R *.lfm}
{$ENDIF}

procedure TMain.WindowMenuItemClick(Sender: TObject);
begin
  MDIChildren[TMenuItem(Sender).Tag].BringToFront;
end;

procedure TMain.EnableDoodleControls(Flag: Boolean);
begin
  sbSave.Enabled:=Flag;
  plPenOptions.Enabled:=Flag;
  sbPrevious.Enabled:=Flag;
  sbNext.Enabled:=Flag;
  if not Flag then ClearWindowMenu else UpdateWindowMenu;
end;

procedure TMain.UpdateActiveDoodle;
begin
  EnableDoodleControls(Assigned(ActiveMDIChild));
  UpdateWindowMenu;
end;

procedure TMain.ClearWindowMenu;
begin
  while mWindow.Count > 4 do mWindow.Items[4].Free;
end;

procedure TMain.UpdateWindowMenu;
var
  I: Integer;
  WindowMenuItem: TMenuItem;
begin
  ClearWindowMenu;
  for I:=0 to MDIChildCount - 1 do begin
    WindowMenuItem:=TMenuItem.Create(Self);
    if I < 9 then
      WindowMenuItem.Caption:='&' + IntToStr(I + 1) + '. ' + MDIChildren[I].Caption
    else
      WindowMenuItem.Caption:=MDIChildren[I].Caption;
    WindowMenuItem.Tag:=I;
    WindowMenuItem.OnClick:=WindowMenuItemClick;
    mWindow.Add(WindowMenuItem);
  end;
end;

procedure TMain.CloseAllWindows;
begin
  while MDIChildCount > 0 do MDIChildren[0].Free;
end;

procedure TMain.mHelpAboutClick(Sender: TObject);
begin
  ShowMessage('MDI Doodle Test Application by Bad Sector');
end;

procedure TMain.sbNewClick(Sender: TObject);
begin
  mFileNew.Click;
end;

procedure TMain.sbOpenClick(Sender: TObject);
begin
  mFileOpen.Click;
end;

procedure TMain.mFileNewClick(Sender: TObject);
begin
  Application.CreateForm(TDoodle, Doodle);
end;

procedure TMain.tmrFirstTimeInitTimer(Sender: TObject);
begin
  tmrFirstTimeInit.Enabled:=False;
  mFileNew.Click;
  UpdateActiveDoodle;
end;

procedure TMain.plColorClick(Sender: TObject);
begin
  ColorDialog1.Color:=plColor.Color;
  if ColorDialog1.Execute then plColor.Color:=ColorDialog1.Color;
end;

procedure TMain.mWindowTileClick(Sender: TObject);
begin
  Tile;
end;

procedure TMain.mWindowCascadeClick(Sender: TObject);
begin
  Cascade;
end;

procedure TMain.mWindowArrangeIconsClick(Sender: TObject);
begin
  ArrangeIcons;
end;

procedure TMain.sbPreviousClick(Sender: TObject);
begin
  Previous;
end;

procedure TMain.sbNextClick(Sender: TObject);
begin
  Next;
end;

procedure TMain.mFileExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMain.mFileOpenClick(Sender: TObject);
begin
  ShowMessage('Assume opening a file is implemented');
end;

procedure TMain.sbSaveClick(Sender: TObject);
begin
  if ActiveMDIChild is TDoodle then
    TDoodle(ActiveMDIChild).mFileSave.Click;
end;

procedure TMain.mViewToolbarClick(Sender: TObject);
begin
  mViewToolbar.Checked:=not mViewToolbar.Checked;
  plToolbar.Visible:=mViewToolbar.Checked;
end;

procedure TMain.mViewStatusClick(Sender: TObject);
begin
  mViewStatus.Checked:=not mViewStatus.Checked;
  StatusBar1.Visible:=mViewStatus.Checked;
end;

procedure TMain.plToolBarClick(Sender: TObject);
begin
  plToolbar.Height:=plToolBar.Height + 4;
end;

procedure TMain.SpeedButton1Click(Sender: TObject);
var
  I: Integer;
begin
  for I:=0 to Menu.Items.Count - 1 do
    ShowMessage(Menu.Items[I].Caption);
end;

end.
